#!/bin/sh

ACTION=${1}
shift 1

get_image() {
  IMAGE_NAME=$(uci get prowlarr.@prowlarr[0].image_name 2>/dev/null)
}

do_install_detail() {
  local config=$(uci get prowlarr.@prowlarr[0].config_path 2>/dev/null)
  local port=$(uci get prowlarr.@prowlarr[0].port 2>/dev/null)
  local IMAGE_NAME=$(uci get prowlarr.@prowlarr[0].image_name 2>/dev/null)
  local username=$(uci get prowlarr.@prowlarr[0].username 2>/dev/null)
  local password=$(uci get prowlarr.@prowlarr[0].password 2>/dev/null)

  GEN_PASS=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)
  GEN_PASS2=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)

  LAN_IP=$(uci get network.lan.ipaddr)
  LAN_IP="${LAN_IP%/*}"

  if [ -z "$config" ]; then
    echo "config path is empty!"
    exit 1
  fi

  [ -z "$port" ] && port=8160
  [ -z "$IMAGE_NAME" ] && IMAGE_NAME=lscr.io/linuxserver/prowlarr:develop

  rm -R /opt/docker2/compose/prowlarr-app
  mkdir /opt/docker2/compose/prowlarr-app

  touch /opt/docker2/compose/prowlarr-app/docker-compose.yml
  cat > /opt/docker2/compose/prowlarr-app/docker-compose.yml <<EOF
version: "2.1"
services:
  prowlarr:
    image: $IMAGE_NAME
    container_name: prowlarr
    environment:
      - PUID=1000
      - PGID=1000
      - TZ=Europe/London
    volumes:
      - ${config}:/config
    ports:
      - ${LAN_IP}:${port}:9696
    restart: unless-stopped
    labels:
      plugsy.name: "Prowlarr"
      plugsy.category: "Home"
      plugsy.icon: "@styled-icons/simple-icons/Prowlarr"
      plugsy.link: "http://${LAN_IP}:${port}"
EOF

  docker-compose -f /opt/docker2/compose/prowlarr-app/docker-compose.yml up -d

  uci add shortcutmenu lists
  uci set shortcutmenu.@lists[-1].webname="$IMAGE_NAME"
  uci set shortcutmenu.@lists[-1].weburl="$LAN_IP:$port"
  uci set shortcutmenu.@lists[-1].webpath="/"
  uci commit shortcutmenu
}

usage() {
  echo "usage: $0 sub-command"
  echo "where sub-command is one of:"
  echo "      install                Install Prowlarr"
  echo "      upgrade                Upgrade Prowlarr"
  echo "      rm/start/stop/restart  Remove/Start/Stop/Restart Prowlarr"
  echo "      status                 Prowlarr status"
  echo "      port                   Prowlarr port"
}

case ${ACTION} in
  "install"|"upgrade")
    get_image
    do_install_detail
  ;;
  "rm")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    echo "Stopping and removing containers..."
    for ID in $CONTAINER_IDS; do
      docker stop $ID
      docker rm $ID
    done
    docker rmi -f $IMAGE_NAME
  ;;
  "start"|"stop"|"restart")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    for ID in $CONTAINER_IDS; do
      docker ${ACTION} ${ID}
    done
  ;;
  "status")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Status}}'
  ;;
  "port")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Ports}}' | grep -om1 '0.0.0.0:[0-9]*' | sed 's/0.0.0.0://'
  ;;
  *)
    usage
    exit 1
  ;;
esac
